#include <string.h>
#include <unistd.h>
#include "sg_lgpio.h"
#include "rc522.h"


//==============================================================================================

int uart_fd  = 0;
void  M500PcdConfigISOType( unsigned char type );          //config work mode
void  PcdReset( void );                       	//reset
char  PcdRequest( unsigned char req_code, unsigned char * pTagType ); //Request
char  PcdAnticoll( unsigned char * pSnr);                 

char  PcdSelect( unsigned char * pSnr );
char  PcdAuthState( unsigned char ucAuth_mode, unsigned char ucAddr, unsigned char * pKey, unsigned char * pSnr );

char  PcdWrite( unsigned char ucAddr, unsigned char * pData );
char  PcdRead( unsigned char ucAddr, unsigned char * pData );

void PcdAntennaOn ( void );
void PcdAntennaOff ( void );

void RC522_Init(void)
{
    Lgpio_export();
    Lgpio_set_pinmode(RST_PIN,LGPIO_OUTPUT);
    Lgpio_set_pinmode(LED_PIN,LGPIO_OUTPUT);
    Lgpio_write_pin(LED_PIN, LGPIO_HIGH);//LED off
    LG_PWM(BUZZER_PIN,1000,0);//set buzzer
    LG_UART_Init("/dev/ttyS0", 9600);
    PcdReset();
    M500PcdConfigISOType ( 'A' );//set work mode to 'A'
}


// the M1 card is divided into 16 sectors, each sector consists of four blocks(block0,block1,block2,block3)
// the 64 blocks of 16 sectors are numbered by absolute address:0~63
// block 0 of 0th sector(ie absolute address 0 block) is used to store the manufacturer code,which has been
// cured and cannot be changed
// block 0,block 1, and block 2 of each sector are data blocks,which can be used to store data
// block 3 of each sector is a control block (absolute address:block 3,block 7,block 11...) including
// password A,access control,password B,etc


unsigned char CT[2];//card type 
unsigned char SN[4]; //card serial number
unsigned char RFID[16];	//svae RFID 
unsigned char total=0;
unsigned char KEY[6]={0xff,0xff,0xff,0xff,0xff,0xff};
unsigned char AUDIO_OPEN[6] = {0xAA, 0x07, 0x02, 0x00, 0x09, 0xBC};
unsigned char RFID1[16]={0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x07,0x80,0x29,0xff,0xff,0xff,0xff,0xff,0xff};

unsigned char status;
unsigned char s=0x08;

unsigned char Uart_ReadWriteByte(unsigned char TxData)
{
    unsigned char rev = 0;
    int timeout = 3;
    int cnt = 0;
    //LG_UART_writebyte(uart_fd, TxData);
    LG_UART_writebyte(TxData);
    while( timeout > 0)
    {
	delay(3);  // when the bandrate is 9600:delay(3)
		    //when the bandrate is 115200:delay(1)
	cnt = lgSerialDataAvailable(UART_handle);
	if (cnt == 1)
	{
	    rev = LG_UART_readbyte();
	    break;
	}
	timeout -= 1;
    }
    return rev;
}

/****************************************************
 * name：WriteRawRC   
 * describe：write RC522 register  
 * input ：ucAddress， ucValue
 */
void WriteRawRC ( unsigned char ucAddress, unsigned char ucValue )
{  
    unsigned char ch = Uart_ReadWriteByte(ucAddress & 0x3F);
    if(ch != ucAddress)
    {
	printf("not equal");
    }
    LG_UART_writebyte(ucValue);
    
}
/**************************************************************************************
 * name：ReadRawRC 
 * read current value from the register
 */
unsigned char ReadRawRC ( unsigned char ucAddress )
{
    return Uart_ReadWriteByte((ucAddress&0x3F) | 0x80);
}
/**************************************************************************************
 * name：ClearBitMask 
 * clear RC522 register bit
 */
void ClearBitMask ( unsigned char ucReg, unsigned char ucMask )  
{
    unsigned char ucTemp;
    ucTemp = ReadRawRC ( ucReg );
    WriteRawRC ( ucReg, ucTemp & ( ~ ucMask) );  // clear bit mask
}
/*********************************************************************
 * name：SetBitMask 
 * set bit for RC522 register
 */
void SetBitMask ( unsigned char ucReg, unsigned char ucMask )  
{
    unsigned char ucTemp;
    ucTemp = ReadRawRC ( ucReg );
    WriteRawRC ( ucReg, ucTemp | ucMask );         // set bit mask
}
/*****************************************************************
 * name：PcdAntennaOn 
 * turn on the antenna
 */
void PcdAntennaOn ( void )
{
    unsigned char uc;
    uc = ReadRawRC ( TxControlReg );
    if ( ! ( uc & 0x03 ) )
	SetBitMask(TxControlReg, 0x03);
}
/**************************************************************
 * name：PcdAntennaOff 
 * turn off antenna
 */
void PcdAntennaOff ( void )
{
    ClearBitMask ( TxControlReg, 0x03 );
}
/**************************************************************************************
 * describe  ：set RC522 work mode   
 * ucType，work mode
 */
void M500PcdConfigISOType ( unsigned char ucType )
{
    if ( ucType == 'A') //ISO14443_A
    {
	ClearBitMask ( Status2Reg, 0x08 );
	WriteRawRC ( ModeReg, 0x3D );//3F
	WriteRawRC ( RxSelReg, 0x86 );//84
	WriteRawRC( RFCfgReg, 0x7F ); //4F
	WriteRawRC( TReloadRegL, 30 );//tmoLength);// TReloadVal = 'h6a =tmoLength(dec) 
	WriteRawRC ( TReloadRegH, 0 );
	WriteRawRC ( TModeReg, 0x8D );
	WriteRawRC ( TPrescalerReg, 0x3E );
	//delayMicroseconds(2);
	usleep(2);
	PcdAntennaOn ();//turn of antenna
    }
}
/**************************************************************************************
 * name ：PcdComMF522
 * describe ：communication between RC522 and ISO14443 card
 * input  ：ucCommand，RC522 command
 *         pInData，the data from RC522 send to the card
 *         ucInLenByte，len for data send
 *         pOutData，datas return
 *         pOutLenBit，data bits return
 * return: MI_OK，success
 */
char PcdComMF522 ( unsigned char ucCommand, unsigned char * pInData, unsigned char ucInLenByte, unsigned char * pOutData, unsigned long * pOutLenBit )		
{
    char cStatus = MI_ERR;
    unsigned char ucIrqEn   = 0x00;
    unsigned char ucWaitFor = 0x00;
    unsigned char ucLastBits;
    unsigned char ucN;
    unsigned long ul;
    switch ( ucCommand )
    {
       case PCD_AUTHENT:  //Mifare authenticate
          ucIrqEn   = 0x12;  //error interrupt request enabled:ErrIEn IdleIEn
          ucWaitFor = 0x10;  //time for Request and authenticate and check the idleIRq bit
          break;
			 
       case PCD_TRANSCEIVE:  //receive and send
          ucIrqEn   = 0x77;  //enable TxIEn RxIEn IdleIEn LoAlertIEn ErrIEn TimerIEn
          ucWaitFor = 0x30;  //time for Request and authenticate and check the RxIRq IdleIRq bit
          break;
			 
       default:
         break;
    }
    WriteRawRC ( ComIEnReg, ucIrqEn | 0x80 );	//IRqInv sets the pin IRQ to the opposite
						//value of the IRq bit of Status1Reg
    ClearBitMask ( ComIrqReg, 0x80 );		//when Set1 bit were cleared，clear CommIRqReg mask bit
    WriteRawRC ( CommandReg, PCD_IDLE );	//write IDLE command
    SetBitMask ( FIFOLevelReg, 0x80 );		//set FlushBuffer to clear the read and write pointers of
						//the internal FIFO and the BufferOvfl flag of ErrReg is cleared
    
    for ( ul = 0; ul < ucInLenByte; ul ++ )
	WriteRawRC ( FIFODataReg, pInData [ ul ] ); //write data to FIFO
			
    WriteRawRC ( CommandReg, ucCommand ); 	//write command
   
    if ( ucCommand == PCD_TRANSCEIVE )
	SetBitMask(BitFramingReg,0x80);  //StartSend is set to start data transmission. The bit
	                                 //is only valid when used with send and receive commands
    
    ul = 1000;//adjusted according to the clock frequency, the maximum
	      //waiting time for operating the M1 card is 25 ms
		
    do 	
    {
         ucN = ReadRawRC ( ComIrqReg );	//check event interrupt
         ul --;
    } 
    while ( ( ul != 0 ) && ( ! ( ucN & 0x01 ) ) && ( ! ( ucN & ucWaitFor ) ) );	
		
    ClearBitMask ( BitFramingReg, 0x80 );	//clear StartSend bit
		
    if ( ul != 0 )
    {
	if ( ! (( ReadRawRC ( ErrorReg ) & 0x1B )) )	//read error flag:register BufferOfI CollErr ParityErr ProtocolErr
	{
	    cStatus = MI_OK;
	    if ( ucN & ucIrqEn & 0x01 )	  //time interrupt occurs
		    cStatus = MI_NOTAGERR;   
	    if ( ucCommand == PCD_TRANSCEIVE )
	    {
		ucN = ReadRawRC ( FIFOLevelReg );   //read len from FIFO save data
		ucLastBits = ReadRawRC ( ControlReg ) & 0x07;	//the number of valid bits of the last received byte
		if ( ucLastBits )
		    * pOutLenBit = ( ucN - 1 ) * 8 + ucLastBits;   //N bytes minus 1 (the last byte) + the number
		                                                   //of valid bits of the last received byte
		else
		    * pOutLenBit = ucN * 8;   	//last received byte whole byte valid
		if ( ucN == 0 )	
		ucN = 1;    
		if ( ucN > MAXRLEN )
		    ucN = MAXRLEN;   
		for ( ul = 0; ul < ucN; ul ++ )
		    pOutData [ ul ] = ReadRawRC ( FIFODataReg );   
	    }		
        }
	else
	    cStatus = MI_ERR;   
	    //printf("ErrorReg=%d\r\n",ErrorReg);
    }
    SetBitMask ( ControlReg, 0x80 );  // stop timer now
    WriteRawRC ( CommandReg, PCD_IDLE ); 
    return cStatus;
}
//================RC522_Handel================================//
bool RC522_READ(unsigned char * str)
{
    status = PcdRequest(PICC_REQALL,CT);//Request card
    if(status==MI_OK)//Request success
    {
         status=MI_ERR;
         status = PcdAnticoll(SN);//Anticoll       
    }
    if (status==MI_OK)//Anticoll success
    {
        status=MI_ERR;		
        status =PcdSelect(SN);
    }
    if(status==MI_OK)//select card success
    {
        status=MI_ERR;
        status =PcdAuthState(0x60,0x09,KEY,SN);
     }
	if(status==MI_OK)//AuthState success
    {
        status=MI_ERR;
        status=PcdRead(s,RFID);
    }
    if(status==MI_OK)//read card success
    {
        status=MI_ERR;
	memset(str,0,16);
	memcpy(str,RFID,16); //get RFID
	return 1;
   }
   
  return 0;
}
//==================RC522_Handel==========================//
bool RC522_WRITE(unsigned char * msg)
{
    if(msg == NULL)return false;
    status = PcdRequest(PICC_REQALL,CT);//Request card
    if(status==MI_OK)//Request success
    {
	status=MI_ERR;
	status = PcdAnticoll(SN);// Anticoll	       
    }
    if (status==MI_OK)// Anticoll success
    {
        status=MI_ERR;		
        status =PcdSelect(SN);
    }
    if(status==MI_OK)//select card success
    {
        status=MI_ERR;
        status =PcdAuthState(0x60,0x09,KEY,SN);
    }
    if(status==MI_OK)//AuthState success
    {
        status=MI_ERR;
	status=PcdWrite(s,msg);
    }
    if(status==MI_OK)//write card success
	return true;
    return false;
}
/****************************************************************************
 * name：PcdRequest 
 * input  ：ucReq_code，Request card mode
 *                   = 0x52，find all 14443A-compliant cards in the induction area
 *                   = 0x26，find a card that is not in sleep mode
 *         pTagType，
 *                   = 0x4400，Mifare_UltraLight
 *                   = 0x0400，Mifare_One(S50)
 *                   = 0x0200，Mifare_One(S70)
 *                   = 0x0800，Mifare_Pro(X))
 *                   = 0x4403，Mifare_DESFire
 * return : MI_OK，success
 */
char PcdRequest ( unsigned char ucReq_code, unsigned char * pTagType )
{
    char cStatus;  
    unsigned char ucComMF522Buf [ MAXRLEN ]; 
    unsigned long ulLen;

    ClearBitMask ( Status2Reg, 0x08 );	//clear up the case in which the MIFARECyptol unit 
					//turned on and all card data communications were encrypted
    WriteRawRC ( BitFramingReg, 0x07 );	//send the seven bits of the last byte
    SetBitMask ( TxControlReg, 0x03 );	//the output signal of the TX1, TX2 pin transmits
					//the modulated 13.56 energy carrier signal
    ucComMF522Buf [ 0 ] = ucReq_code;	//store card commmand word
    cStatus = PcdComMF522( PCD_TRANSCEIVE,ucComMF522Buf, 1, ucComMF522Buf, & ulLen );	//Request  
    if ( ( cStatus == MI_OK ) && ( ulLen == 0x10 ) )	//Request success return card type
    {    
       * pTagType = ucComMF522Buf [ 0 ];
       * ( pTagType + 1 ) = ucComMF522Buf [ 1 ];
    }
    else
       cStatus = MI_ERR;
    return cStatus;
}

/*****************************************************************
 * name：PcdAnticoll pSnr，card serial number，4 bytes
 * return: MI_OK，sccess
 */
char PcdAnticoll ( unsigned char * pSnr )
{
    char cStatus;
    unsigned char uc, ucSnr_check = 0;
    unsigned char ucComMF522Buf [ MAXRLEN ]; 
    unsigned long ulLen;
    ClearBitMask ( Status2Reg, 0x08 );	//clear the MFCryptol On bit This bit can only be set
					//after a successful MFAuthent command is executed
    WriteRawRC ( BitFramingReg, 0x00);	//clear register  stop send and 
    ClearBitMask ( CollReg, 0x80 );	//clear ValuesAfterColl all received bits are cleared after collision
    ucComMF522Buf [ 0 ] = 0x93;	//Anticoll command
    ucComMF522Buf [ 1 ] = 0x20;
    cStatus = PcdComMF522 ( PCD_TRANSCEIVE, ucComMF522Buf, 2, ucComMF522Buf, & ulLen);//communicate to the card
    if ( cStatus == MI_OK)		//communition success
    {
	for ( uc = 0; uc < 4; uc ++ )
        {
            * ( pSnr + uc )  = ucComMF522Buf [ uc ];//read card uid
            ucSnr_check ^= ucComMF522Buf [ uc ];
        }	
        if ( ucSnr_check != ucComMF522Buf [ uc ] )
	    cStatus = MI_ERR;    		 
    }
    SetBitMask ( CollReg, 0x80 );
    return cStatus;
}
/************************************************************************************************************
 * name：CalulateCRC
 * describe  ：calculate CRC16 with RC522
 * input  ：pIndata，calculate the CRC16 of the array
 *         ucLen，the length of CRC16 array
 *         pOutData，the first address where the calculation result is stored
 */
void CalulateCRC ( unsigned char * pIndata, unsigned char ucLen, unsigned char * pOutData )
{
    unsigned char uc, ucN;
    ClearBitMask(DivIrqReg,0x04);
    WriteRawRC(CommandReg,PCD_IDLE);
    SetBitMask(FIFOLevelReg,0x80);
    for ( uc = 0; uc < ucLen; uc ++)
	WriteRawRC ( FIFODataReg, * ( pIndata + uc ) );   
    WriteRawRC ( CommandReg, PCD_CALCCRC );
    uc = 0xFF;
    do 
    {
        ucN = ReadRawRC ( DivIrqReg );
        uc --;
    } while ( ( uc != 0 ) && ! ( ucN & 0x04 ) );
		
    pOutData [ 0 ] = ReadRawRC ( CRCResultRegL );
    pOutData [ 1 ] = ReadRawRC ( CRCResultRegM );
}

/*****************************************************************
 * name：PcdSelect
 * describe  ：select card
 * input  ：pSnr，card serial number，4 bytes
 * retuen  : status
 *          = MI_OK，success
 */
char PcdSelect ( unsigned char * pSnr )
{
    char ucN;
    unsigned char uc;
    unsigned char ucComMF522Buf [ MAXRLEN ]; 
    unsigned long  ulLen;
    ucComMF522Buf [ 0 ] = PICC_ANTICOLL1;
    ucComMF522Buf [ 1 ] = 0x70;
    ucComMF522Buf [ 6 ] = 0;
    for ( uc = 0; uc < 4; uc ++ )
    {
    	ucComMF522Buf [ uc + 2 ] = * ( pSnr + uc );
    	ucComMF522Buf [ 6 ] ^= * ( pSnr + uc );
    }	
    CalulateCRC ( ucComMF522Buf, 7, & ucComMF522Buf [ 7 ] );
    ClearBitMask ( Status2Reg, 0x08 );
    ucN = PcdComMF522 ( PCD_TRANSCEIVE, ucComMF522Buf, 9, ucComMF522Buf, & ulLen );
    if ( ( ucN == MI_OK ) && ( ulLen == 0x18 ) )
      ucN = MI_OK;  
    else
      ucN = MI_ERR;    
    return ucN;
}
/***********************************************************************
 * name：PcdAuthState authenticate card key
 * input  ：ucAuth_mode，key authenticate mode
 *                     = 0x60，authenticate A key
 *                     = 0x61，authenticate B key
 *         unsigned char ucAddr，block address
 *         pKey，key
 *         pSnr，card serial number,bytes 
 * return  : MI_OK，success
 */
char PcdAuthState ( unsigned char ucAuth_mode, unsigned char ucAddr, unsigned char * pKey, unsigned char * pSnr )
{
    char cStatus;
    unsigned char uc, ucComMF522Buf [ MAXRLEN ];
    unsigned long ulLen;
    ucComMF522Buf [ 0 ] = ucAuth_mode;
    ucComMF522Buf [ 1 ] = ucAddr;
    for ( uc = 0; uc < 6; uc ++ )
	ucComMF522Buf [ uc + 2 ] = * ( pKey + uc );   
    for ( uc = 0; uc < 4; uc ++ )
	ucComMF522Buf [ uc + 8 ] = * ( pSnr + uc );   
    cStatus = PcdComMF522 ( PCD_AUTHENT, ucComMF522Buf, 12, ucComMF522Buf, & ulLen );
    if ( ( cStatus != MI_OK ) || ( ! ( ReadRawRC ( Status2Reg ) & 0x08 ) ) )
    {
	cStatus = MI_ERR; 
    }
    return cStatus;
}

/**********************************************************************************************
 * name：PcdWrite write pData to M1 card block，pData，16 bytes
 * return  : MI_OK，success
 */
char PcdWrite ( unsigned char ucAddr, unsigned char * pData )
{
    char cStatus;
    unsigned char uc, ucComMF522Buf [ MAXRLEN ];
    unsigned long ulLen;
	
    ucComMF522Buf [ 0 ] = PICC_WRITE;
    ucComMF522Buf [ 1 ] = ucAddr;
    CalulateCRC ( ucComMF522Buf, 2, & ucComMF522Buf [ 2 ] );
    cStatus = PcdComMF522 ( PCD_TRANSCEIVE, ucComMF522Buf, 4, ucComMF522Buf, & ulLen );
    if ( ( cStatus != MI_OK ) || ( ulLen != 4 ) || ( ( ucComMF522Buf [ 0 ] & 0x0F ) != 0x0A ) )
         cStatus = MI_ERR;   
    if ( cStatus == MI_OK )
    {
	memcpy(ucComMF522Buf, pData, 16);
	for ( uc = 0; uc < 16; uc ++ )
	    ucComMF522Buf [ uc ] = * ( pData + uc );  
	CalulateCRC ( ucComMF522Buf, 16, & ucComMF522Buf [ 16 ] );
	cStatus = PcdComMF522 ( PCD_TRANSCEIVE, ucComMF522Buf, 18, ucComMF522Buf, & ulLen );
	if ( ( cStatus != MI_OK ) || ( ulLen != 4 ) || ( ( ucComMF522Buf [ 0 ] & 0x0F ) != 0x0A ) )
	    cStatus = MI_ERR;   
    } 
    return cStatus;
}
/****************************************************************
 * name：PcdRead
 * describe:read block data from M1 card,read address is s(0x08)
 * return : MI_OK，success
 */
char PcdRead ( unsigned char ucAddr, unsigned char * pData )
{
    char cStatus;
    unsigned char uc, ucComMF522Buf [ MAXRLEN ]; 
    unsigned long ulLen;

    ucComMF522Buf [ 0 ] = PICC_READ;
    ucComMF522Buf [ 1 ] = ucAddr;
    CalulateCRC ( ucComMF522Buf, 2, & ucComMF522Buf [ 2 ] );
    cStatus = PcdComMF522 (PCD_TRANSCEIVE, ucComMF522Buf, 4, ucComMF522Buf, & ulLen);
    if ( ( cStatus == MI_OK ) && ( ulLen == 0x90 ) )
    {
	for ( uc = 0; uc < 16; uc ++ )
        * ( pData + uc ) = ucComMF522Buf [ uc ];   
    }
    else
        cStatus = MI_ERR;   
    return cStatus;
}

/****************************************************************
 * name：PcdReset 
 * describe: reset RC522 
 */
void PcdReset ( void )
{
    Lgpio_write_pin(RST_PIN, LGPIO_LOW);
    usleep(4);
    Lgpio_write_pin(RST_PIN, LGPIO_HIGH);
    usleep(2);
    WriteRawRC ( CommandReg, 0x0f ); 	//reset the rc522
    while( ReadRawRC ( CommandReg ) & 0x10 );
    ReadRawRC ( CommandReg );
    //delayMicroseconds(5);
    usleep(5);
    WriteRawRC ( ModeReg, 0x3D );	//define the common mode of sending and receiving,
					//communicate with Mifare card,the initial value of CRC 
    WriteRawRC ( TReloadRegL, 30 );  	//low bit for 16-bit timer    
    WriteRawRC ( TReloadRegH, 0 );	//high bit for 16-bit timer
    WriteRawRC ( TModeReg, 0x8D );	//define the settings for the internal timer
    WriteRawRC ( TPrescalerReg, 0x3E );	//set the timer division factor
    WriteRawRC ( TxAutoReg, 0x40 );	//the modulated transmit signal set to 100%ASK	
}








